function [b,w,dopt] = mmse_uec(h,dmin,dmax,nu,Lw,Rn)
% [b,w,dopt] = mmse_uec(h,dmin,dmax,nu,Lw,Rn)
%     b = target impulse response (TIR)
%     w = TEQ
%     dopt = optimum delay (start indexing at 1, not zero)
%     h = transmission channel
%     dmin = lowest delay to search (1 or more)
%     dmax = highest delay to search (Lc-nu-1 or less)
%        ("Delta" in the paper equals (d-1))
%     nu = cyclic prefix length = window length - 1
%     Lw = TEQ length - 1
%     Rn = noise autocorrelation of size (Lw+1)x(Lw+1)
%
%     Computes MMSE TIR and TEQ using the unit energy
%     constraint on the TIR.
%     Assumption: source signal is white with unit variance.
%
%     Written by Rick Martin, 09/18/2002
%     Last updated 09/18/2002
%     Cornell University Blind Equalization Research Group

ds = [dmin:dmax];
Ld = length(ds);
h = h(:);
H = convmtx(h,Lw+1);
HHRi = pinv(H.' * H + Rn);

mmsevec = zeros(Ld,1);
bmtx = zeros(nu+1,Ld);

for di = 1:Ld
   d = ds(di);
   Hwin = H(d:d+nu,:);
   Rd = eye(nu+1) - Hwin*HHRi*Hwin.';
   [mmsevec(di),bmtx(:,di)] = mineig(Rd);
end

[dummy,di_opt] = min(mmsevec);
b = bmtx(:,di_opt);
dopt = ds(di_opt);

% compute TEQ via:
% Rrx b = Rr w
% (Rr)^(-1) = HHRi
% Rrx = Hwin.'

Hwin = H(dopt:dopt+nu,:);
w = HHRi * Hwin.' * b;
